/************************************************************************
 *
 * Copyright 2024 J David Eisenberg
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not
 * use this file except in compliance with the License. You may obtain a copy
 * of the License at http://www.apache.org/licenses/LICENSE-2.0.
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 ************************************************************************/

import java.io.IOException;


import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import org.odftoolkit.odfdom.doc.OdfTextDocument;
import org.odftoolkit.odfdom.dom.OdfContentDom;
import org.odftoolkit.odfdom.dom.OdfStylesDom;

import org.odftoolkit.odfdom.incubator.doc.office.OdfOfficeAutomaticStyles;
import org.odftoolkit.odfdom.incubator.doc.office.OdfOfficeStyles;

import org.odftoolkit.odfdom.dom.element.office.OfficeTextElement;
import org.odftoolkit.odfdom.dom.element.office.OfficeStylesElement;
import org.odftoolkit.odfdom.incubator.doc.text.OdfTextParagraph;

import org.odftoolkit.odfdom.incubator.doc.text.OdfTextHeading;
import org.odftoolkit.odfdom.incubator.doc.text.OdfTextSpan;

import org.odftoolkit.odfdom.incubator.doc.style.OdfDefaultStyle;
import org.odftoolkit.odfdom.incubator.doc.style.OdfStyle;
import org.odftoolkit.odfdom.dom.element.style.StyleParagraphPropertiesElement;

import org.odftoolkit.odfdom.dom.element.style.StyleTextPropertiesElement;
import org.odftoolkit.odfdom.dom.element.OdfStyleBase;

// import org.odftoolkit.odfdom.pkg.NamespaceNames;
import org.odftoolkit.odfdom.dom.style.OdfStyleFamily;

import org.odftoolkit.odfdom.dom.element.style.StyleTabStopsElement;
import org.odftoolkit.odfdom.dom.element.style.StyleTabStopElement;
/**
 *
 * @author J David Eisenberg
 */
public class MakeMovieDocument {

    String inputFileName;
    Document inputDocument;
    XPath inputXPath;    // use this for extracting information from input file

    String outputFileName;
    OdfTextDocument outputDocument;
    OdfContentDom contentDom;    // the document object model for content.xml
    OdfStylesDom stylesDom;    // the document object model for styles.xml

    // the office:automatic-styles element in content.xml
    // this is here for the sake of completeness; this program doesn't use it
    OdfOfficeAutomaticStyles contentAutoStyles;

    // the office:styles element in styles.xml
    OdfOfficeStyles stylesOfficeStyles;

    // the office:text element in the content.xml file
    OfficeTextElement officeText;

    public void run(String[] args)
    {
         if (args.length == 2)
        {
            inputFileName = args[0];
            outputFileName = args[1];

            parseInputFile();
            setupOutputDocument();

            if (outputDocument != null && inputDocument != null)
            {
                cleanOutDocument();

                addOfficeStyles();

                processInputDocument();

                saveOutputDocument();
            }
        }
        else
        {
            System.err.println("Usage: MakeMovieDocument infile outfile");
        }
    }

    void parseInputFile()
    {
        DocumentBuilder builder = null;
        inputDocument = null;

        try {
            inputXPath = XPathFactory.newInstance().newXPath();
            builder = DocumentBuilderFactory.newInstance().newDocumentBuilder();
            inputDocument = builder.parse(inputFileName);
        }
        catch (IOException e)
        {
            System.err.println("Unable to read input file.");
            System.err.println(e.getMessage());
        }
        catch (Exception e)
        {
            System.err.println("Unable to parse input file.");
            System.err.println(e.getMessage());
        }
    }

    void setupOutputDocument()
    {
        try
        {
            outputDocument =
                OdfTextDocument.newTextDocument();
            contentDom = outputDocument.getContentDom();
            stylesDom = outputDocument.getStylesDom();
            contentAutoStyles = contentDom.getOrCreateAutomaticStyles();
            stylesOfficeStyles = outputDocument.getOrCreateDocumentStyles();
            officeText = outputDocument.getContentRoot();
        }
        catch (Exception e)
        {
            System.err.println("Unable to create output file.");
            System.err.println(e.getMessage());
            outputDocument = null;
        }
    }

    void setFontWeight(OdfStyleBase style, String value)
    {
        style.setProperty(StyleTextPropertiesElement.FontWeight, value);
        style.setProperty(StyleTextPropertiesElement.FontWeightAsian, value);
        style.setProperty(StyleTextPropertiesElement.FontWeightComplex, value);
    }

    void setFontStyle(OdfStyleBase style, String value)
    {
        style.setProperty(StyleTextPropertiesElement.FontStyle, value);
        style.setProperty(StyleTextPropertiesElement.FontStyleAsian, value);
        style.setProperty(StyleTextPropertiesElement.FontStyleComplex, value);
    }

    void setFontSize(OdfStyleBase style, String value)
    {
        style.setProperty(StyleTextPropertiesElement.FontSize, value);
        style.setProperty(StyleTextPropertiesElement.FontSizeAsian, value);
        style.setProperty(StyleTextPropertiesElement.FontSizeComplex, value);
    }

    void addOfficeStyles()
    {
        OdfDefaultStyle defaultStyle;
        OdfStyle style;
        StyleParagraphPropertiesElement pProperties;

        StyleTabStopsElement tabStops;
        StyleTabStopElement tabStop;

        // Set default font size to 10 point
        defaultStyle = stylesOfficeStyles.getDefaultStyle(
            OdfStyleFamily.Paragraph);
        setFontSize(defaultStyle, "10pt");

        // movie heading: 14pt, bold
        style = stylesOfficeStyles.newStyle("Movie_20_Heading",
            OdfStyleFamily.Paragraph);
        style.setStyleDisplayNameAttribute("Movie Heading");
        style.setProperty(StyleParagraphPropertiesElement.MarginTop, "0.25cm");
        style.setProperty(StyleParagraphPropertiesElement.MarginBottom, "0.25cm");
        setFontWeight(style, "bold");
        setFontSize(style, "14pt");

        // span for stars: 12pt, bold
        style = stylesOfficeStyles.newStyle("Star_20_Span",
            OdfStyleFamily.Text);
        style.setStyleDisplayNameAttribute("Star Span");
        setFontWeight(style, "bold");
        setFontSize(style, "12pt");

        // Cast heading is 12pt italic
        style = stylesOfficeStyles.newStyle("Cast_20_Heading",
            OdfStyleFamily.Paragraph);
         style.setStyleDisplayNameAttribute("Cast Heading");
        style.setProperty(StyleParagraphPropertiesElement.MarginTop, "0.5cm");
        style.setProperty(StyleParagraphPropertiesElement.MarginBottom, "0.25cm");
        setFontStyle(style, "italic");
        setFontSize(style, "12pt");

        // Paragraph with tab stop at 7.5cm with a
        // leader of "."  This is used for the
        // cast list.
        style = stylesOfficeStyles.newStyle("Cast_20_Para",
            OdfStyleFamily.Paragraph);
        style.setStyleDisplayNameAttribute("Cast Para");
        style.setStyleFamilyAttribute(OdfStyleFamily.Paragraph.toString());

        // build hierarchy from "inside out"
        tabStop = new StyleTabStopElement(stylesDom);
        tabStop.setStylePositionAttribute("7.5cm");
        tabStop.setStyleLeaderStyleAttribute("dotted");
        tabStop.setStyleLeaderTextAttribute(".");
        tabStop.setStyleTypeAttribute("right");

        tabStops = new StyleTabStopsElement(stylesDom);
        tabStops.appendChild(tabStop);

        pProperties = new StyleParagraphPropertiesElement(stylesDom);
        pProperties.appendChild(tabStops);

        style.appendChild(pProperties);

        // style for the movie synopsis
        style = stylesOfficeStyles.newStyle("Synopsis_20_Para",
            OdfStyleFamily.Paragraph);
        style.setStyleDisplayNameAttribute("Synopsis Para");
        style.setProperty(StyleParagraphPropertiesElement.Border,
            "0.035cm solid #000000");
        style.setProperty(StyleParagraphPropertiesElement.Padding, "0.25cm");
        style.setProperty(StyleParagraphPropertiesElement.MarginLeft, "1cm");
        style.setProperty(StyleParagraphPropertiesElement.MarginRight, "1cm");
        style.setProperty(StyleParagraphPropertiesElement.TextIndent, "0.25cm");
 }

    /*
     * The default document has some content in it already (in the case
     * of a text document, a <text:p>.  Clean out all the old stuff.
     */
    void cleanOutDocument()
    {
        Node childNode;

        childNode = officeText.getFirstChild();
        while (childNode != null)
        {
            officeText.removeChild(childNode);
            childNode = officeText.getFirstChild();
        }
    }

    void processInputDocument()
    {
        NodeList movieList;

        movieList = inputDocument.getElementsByTagName("movie");
        for (int i = 0; i < movieList.getLength(); i++)
        {
            try
            {
                processTitle(movieList.item(i));
                processSynopsis(movieList.item(i));
                processCast(movieList.item(i));

            }
            catch(XPathExpressionException e)
            {
                e.printStackTrace(System.err);
            }
        }
    }

    /*
     * Insert the title and star rating into the output document
     */
    void processTitle(Node movieNode) throws XPathExpressionException
    {
        String title;
        String rating;
        OdfTextHeading heading;
        OdfTextSpan stars;

        title = inputXPath.evaluate("heading/title", movieNode);
        rating = inputXPath.evaluate("heading/rating", movieNode);

        heading = new OdfTextHeading(contentDom);
        heading.addStyledContent("Movie_20_Heading", title + " ");

        stars = new OdfTextSpan(contentDom);
        heading.addStyledSpan("Star_20_Span", rating);

        officeText.appendChild(heading);
    }

    /*
     * Insert the movie synopsis into the output document
     */
    void processSynopsis(Node movieNode) throws XPathExpressionException
    {
        NodeList paragraphs;
        OdfTextParagraph paragraph;

        paragraphs = (NodeList) inputXPath.evaluate("synopsis/para", movieNode,
                XPathConstants.NODESET);
        for (int i = 0; i < paragraphs.getLength(); i++)
        {
            paragraph = new OdfTextParagraph(contentDom, "Synopsis_20_Para");
            paragraph.addContent(
                paragraphs.item(i).getFirstChild().getNodeValue());
            officeText.appendChild(paragraph);
        }
    }

    /*
     * Insert the movie synopsis into the output document
     */
    void processCast(Node movieNode) throws XPathExpressionException
    {
        NodeList actors;
        OdfTextParagraph actor;
        String name;
        String role;

        OdfTextHeading heading;

        heading = new OdfTextHeading(contentDom, "Cast_20_Heading");
        heading.appendChild(contentDom.createTextNode("The Cast"));
        officeText.appendChild(heading);

        actors = (NodeList) inputXPath.evaluate("credits/actors/actor",
            movieNode, XPathConstants.NODESET);
        for (int i = 0; i < actors.getLength(); i++)
        {
            actor = new OdfTextParagraph(contentDom, "Cast_20_Para");
            name = inputXPath.evaluate("name", actors.item(i));
            role = inputXPath.evaluate("role", actors.item(i));

            actor.addContent(name);
            if (role != null && !role.equals(""))
            {
                actor.addContentWhitespace("\t" + role);
            }
            officeText.appendChild(actor);
        }
    }

    void saveOutputDocument()
    {
        try
        {
            outputDocument.save(outputFileName);
        }
        catch (Exception e)
        {
            System.err.println("Unable to save document.");
            System.err.println(e.getMessage());
        }
    }

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        MakeMovieDocument app = new MakeMovieDocument();
        app.run(args);
    }

}
